#include<SPI.h>
//The csPin commandlines declares which output ports on the adafruit microcontroller the chip select pins on the digipots will recieve the signal from.
//It is important that it is a digital output port that is chosen, but which ones that are chosen are completely up to the user.
const int csPin1=10;
const int csPin2=11;
const int csPin3=12;
const int csPin4=13;
// Defines which input pins shall recieve signal from the sensors (Recommended to use analog pins)
// It's important that the name matches the inputsignal so it is easy to comprehend the outputdata
int Torque=A5;
int Xaxis=A4;
int Yaxis=A3;
int Zaxis=A2;
unsigned long time;
int counter=0; 

 
const int numReadings = 10;     // the number of readings the program uses to average
int Torque_value[numReadings];  // the measurement from the torque sensor
int Xaxis_value[numReadings];   // the measurement from the x axis of the force sensor
int Yaxis_value[numReadings];   // the measurement from the y axis of the force sensor
int Zaxis_value[numReadings];   // the measurement from the z axis of the force sensor
int readIndex = 0;              // the index of the current measurement  
int Torque_total=0;   //The total of the measurements from the torque sensor
int Xaxis_total=0;    //The total of the measurements from the x axis of the force sensor
int Yaxis_total=0;    //The total of the measurements from the y axis of the force sensor
int Zaxis_total=0;    //The total of the measurements from the z axis of the force sensor
int Torque_average=0; //Holds the average of the measurements from the torque sensor
int Xaxis_average=0;  //Holds the average of the measurements from the x axis of the force sensor
int Yaxis_average=0;  //Holds the average of the measurements from the y axis of the force sensor
int Zaxis_average=0;  //Holds the average of the measurements from the z axis of the force sensor
             
void setup() {
  Serial.begin(9600);
  pinMode(csPin1, OUTPUT);
  pinMode(csPin2, OUTPUT);
  pinMode(csPin3, OUTPUT);
  pinMode(csPin4, OUTPUT);
  pinMode(Torque, INPUT_PULLUP);
  pinMode(Xaxis, INPUT_PULLUP);
  pinMode(Yaxis, INPUT_PULLUP);
  pinMode(Zaxis, INPUT_PULLUP);
  analogReadResolution(12);
//SPI.begin initialises SPI communication with the digipots. 
  SPI.begin(); 
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE0);
//Resets the the measurements registers before running the program
  for (int thisReading = 0; thisReading < numReadings; thisReading++) {
    Torque_value[thisReading] = 0;
    Xaxis_value[thisReading] = 0;
    Yaxis_value[thisReading] = 0;
    Zaxis_value[thisReading] = 0;
  }
  while(!Serial);
}

void loop() {
  //Sets the resistance value for the digipots by changing the wiper position based on the middle inputvalue
  //To find which wiper position gives which resistancevalue, take the total potmeter value and divide it by 255.
  //PS! To calculate which gain requires which resistance value, it is possible to use the formula Resistance=200000/(Gain-5)
  DigitalPotWrite(0x00, 1, csPin1);
  DigitalPotWrite(0x00, 1, csPin2);
  DigitalPotWrite(0x00, 1, csPin3);
  DigitalPotWrite(0x00, 1, csPin4);
  
  // subtract the last reading from the arrays after it has been filled once
  if (counter>=10){
  Torque_total = Torque_total - Torque_value[readIndex];
  Xaxis_total = Xaxis_total - Xaxis_value[readIndex];
  Yaxis_total= Yaxis_total - Yaxis_value[readIndex];
  Zaxis_total= Zaxis_total - Zaxis_value[readIndex];
  }
  // Read from the sensors:
  Torque_value[readIndex] = analogRead(Torque);
  Xaxis_value[readIndex] = analogRead(Xaxis);
  Yaxis_value[readIndex] = analogRead(Yaxis);
  Zaxis_value[readIndex] = analogRead(Zaxis);
  // Add the readings to the totals:
  Torque_total = Torque_total + Torque_value[readIndex];
  Xaxis_total = Xaxis_total + Xaxis_value[readIndex];
  Yaxis_total = Yaxis_total + Yaxis_value[readIndex];
  Zaxis_total = Zaxis_total + Zaxis_value[readIndex];
  // Advance to the next position in the array:
  readIndex = readIndex + 1;
  // Resets the readIndex when at the end of the array: 
  if (readIndex >= numReadings) {
    readIndex = 0;
  }
  // calculates the averages:
  Torque_average = Torque_total / numReadings;
  Xaxis_average = Xaxis_total / numReadings;
  Yaxis_average = Yaxis_total / numReadings;
  Zaxis_average = Zaxis_total / numReadings;
  // Changes the average value from an int to a float
  float Torque_float=float(Torque_average);
  float Xaxis_float=float(Xaxis_average);
  float Yaxis_float=float(Yaxis_average);
  float Zaxis_float=float(Zaxis_average);
  // Remaps the siganl from 0-4095 to +-50 newton or newton millimeters. 
  float Torque_new_average=(Torque_float/4095.00)*10000.00-5000.00;
  float Xaxis_new_average=(Xaxis_float/4095.00)*100.00-50;
  float Yaxis_new_average=(Yaxis_float/4095.00)*100.00-50;
  float Zaxis_new_average=(Zaxis_float/4095.00)*100.00-50;
  //sends the averages to the computer to be plotted:
  time=millis();
  Serial.print(time);
  Serial.print(" ");
  Serial.print(Torque_new_average);
  Serial.print(" "); 
  Serial.print(Xaxis_new_average);
  Serial.print(" ");
  Serial.print(Yaxis_new_average);
  Serial.print(" ");
  Serial.println(Zaxis_new_average);
  // delay in between reads for stability
  delay(1);
  // Counts upwards to ten to start the value subtraction loop at the top
  if (counter<=10){
  counter=counter+1;
  }
}
//Code that uses SPI to send a posistionvalue from 0-255 to the digipots. 
void DigitalPotWrite(int cmd, int val, int CS_PIN)
{
  // constrain input value within 0 - 255
  val = constrain(val, 0, 255);
  // set the CS pin to low to select the chip and allow transfer:
  digitalWrite(CS_PIN, LOW);
  // send the command and value via SPI:
  SPI.transfer(cmd);
  SPI.transfer(val);
  // Set the CS pin high to execute the command:
  digitalWrite(CS_PIN, HIGH);
}
